@@ -1,10 +1,7 @@
 # Client Core Class
 package API::Client::Core;
 
-use namespace::autoclean -except => 'has';
-
 use Data::Object::Class;
-use Data::Object::Class::Syntax;
 use Data::Object::Signatures;
 
 use Data::Object::Library qw(
@@ -12,39 +9,57 @@ use Data::Object::Library qw(
     Int
 );
 
-use API::Client::Exception;
-
 use Mojo::Transaction;
 use Mojo::UserAgent;
 use Mojo::URL;
 
-our $VERSION = '0.02'; # VERSION
+use API::Client::Exception;
+
+our $VERSION = '0.03'; # VERSION
 
 # ATTRIBUTES
 
-has debug      => rw;
-has fatal      => rw;
-has retries    => rw;
-has timeout    => rw;
-has url        => ro;
-has user_agent => ro;
+has debug => (
+    is       => 'rw',
+    isa      => Int,
+    default  => 0,
+    required => 0,
+);
 
-# CONSTRAINTS
+has fatal => (
+    is       => 'rw',
+    isa      => Int,
+    default  => 0,
+    required => 0,
+);
 
-opt debug      => Int;
-opt fatal      => Int;
-opt retries    => Int;
-opt timeout    => Int;
-opt url        => InstanceOf['Mojo::URL'];
-opt user_agent => InstanceOf['Mojo::UserAgent'];
+has retries => (
+    is       => 'rw',
+    isa      => Int,
+    default  => 0,
+    required => 0,
+);
 
-# DEFAULTS
+has timeout => (
+    is       => 'rw',
+    isa      => Int,
+    default  => 10,
+    required => 0,
+);
 
-def debug      => 0;
-def fatal      => 0;
-def retries    => 0;
-def timeout    => 10;
-def user_agent => method { Mojo::UserAgent->new };
+has url => (
+    is       => 'ro',
+    isa      => InstanceOf['Mojo::URL'],
+    coerce   => fun { ref($_[0]) ? $_[0] : Mojo::URL->new($_[0]) },
+    required => 0,
+);
+
+has user_agent => (
+    is       => 'ro',
+    isa      => InstanceOf['Mojo::UserAgent'],
+    default  => fun { Mojo::UserAgent->new },
+    required => 0,
+);
 
 # DELEGATES
 
@@ -59,6 +74,7 @@ my @methods = qw(
 );
 
 around [@methods] => fun ($orig, $self, %args) {
+
     my $retries = $self->retries;
     my $ua      = $self->user_agent;
 
@@ -67,24 +83,29 @@ around [@methods] => fun ($orig, $self, %args) {
     $ua->connect_timeout($self->timeout);
     $ua->request_timeout($self->timeout);
 
-    # request defaults
+    # request initialization
     $ua->on(start => fun ($ua, $tx) {
+
         $self->PREPARE($ua, $tx, %args);
+
     });
 
     # retry entry point
     RETRY:
 
     # execute transaction
-    my $tx  = $self->$orig(%args);
-    my $ok  = $tx->res->code !~ /(4|5)\d\d/;
+    my $tx = $self->$orig(%args);
 
     # fetch transaction objects
     my $req = $tx->req;
     my $res = $tx->res;
 
+    # determine success/failure
+    my $ok = $res->code ? $res->code !~ /(4|5)\d\d/ : 0;
+
     # attempt logging where applicable
-    if ($self->debug) {
+    if ($req and $res and $self->debug) {
+
         my $reqstr = $req->to_string;
         my $resstr = $res->to_string;
 
@@ -93,28 +114,33 @@ around [@methods] => fun ($orig, $self, %args) {
 
         print STDOUT $reqstr;
         print STDOUT $resstr;
+
     }
 
     # retry transaction where applicable
     goto RETRY if $retries-- > 0 and not $ok;
 
     # throw exception if fatal is enabled
-    if ($self->fatal and not $ok) {
+    if ($req and $res and $self->fatal and not $ok) {
+
         API::Client::Exception->throw(
             tx     => $tx,
             code   => $res->code,
             method => $req->method,
             url    => $req->url,
         );
+
     }
 
     # return JSON
     return $res->json;
+
 };
 
 # METHODS
 
 method DELETE (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
+
     my $ua  = $self->user_agent;
     my $url = $self->url->clone;
 
@@ -122,13 +148,17 @@ method DELETE (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
     $url->query($url->query->merge(%$query)) if keys %$query;
 
     return $ua->delete($url, ({}, keys(%$data) ? (json => $data) : ()));
+
 }
 
 fun DESTROY {
+
     ; # Protect subclasses using AUTOLOAD
+
 }
 
 method GET (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
+
     my $ua  = $self->user_agent;
     my $url = $self->url->clone;
 
@@ -136,9 +166,11 @@ method GET (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
     $url->query($url->query->merge(%$query)) if keys %$query;
 
     return $ua->get($url, ({}, keys(%$data) ? (json => $data) : ()));
+
 }
 
 method HEAD (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
+
     my $url = $self->url->clone;
     my $ua  = $self->user_agent;
 
@@ -146,9 +178,11 @@ method HEAD (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
     $url->query($url->query->merge(%$query)) if keys %$query;
 
     return $ua->head($url, ({}, keys(%$data) ? (json => $data) : ()));
+
 }
 
 method OPTIONS (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
+
     my $url = $self->url->clone;
     my $ua  = $self->user_agent;
 
@@ -156,9 +190,11 @@ method OPTIONS (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
     $url->query($url->query->merge(%$query)) if keys %$query;
 
     return $ua->options($url, ({}, keys(%$data) ? (json => $data) : ()));
+
 }
 
 method PATCH (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
+
     my $url = $self->url->clone;
     my $ua  = $self->user_agent;
 
@@ -166,9 +202,11 @@ method PATCH (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
     $url->query($url->query->merge(%$query)) if keys %$query;
 
     return $ua->patch($url, ({}, keys(%$data) ? (json => $data) : ()));
+
 }
 
 method POST (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
+
     my $url = $self->url->clone;
     my $ua  = $self->user_agent;
 
@@ -176,9 +214,11 @@ method POST (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
     $url->query($url->query->merge(%$query)) if keys %$query;
 
     return $ua->post($url, ({}, keys(%$data) ? (json => $data) : ()));
+
 }
 
 method PUT (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
+
     my $url = $self->url->clone;
     my $ua  = $self->user_agent;
 
@@ -186,6 +226,7 @@ method PUT (Str :$path = '', HashRef :$data = {}, HashRef :$query = {}) {
     $url->query($url->query->merge(%$query)) if keys %$query;
 
     return $ua->put($url, ({}, keys(%$data) ? (json => $data) : ()));
+
 }
 
 1;
